'use strict';

var GitHubApi = require('github');
var shell = require('shelljs');
var Download = require('download');
var Registry = require('npm-registry-client');

var npm = new Registry({});
var baseUri = 'https://registry.npmjs.org/';
var params = {
  timeout: 1000
};
var github = new GitHubApi({
  version: '3.0.0'
});

module.exports = {
  execute: execute,
  executeForTest: executeForTest
};

function execute(config) {
  var console = config.console;

  console.info('Fetching boilerplate metadata');

  var op = config.latest ? initLatest : initFromNpm;

  return op(config).then(processTarball.bind(null, console, config));
}

function executeForTest(config) {
  var op = config.latest ? initLatest : initFromNpm;
  return op(config);
}

function initLatest(config) {
  return new Promise(function (resolve, reject) {
    var uri = baseUri + config.boilerplate + '/latest';
    npm.get(uri, params, function (err, data) {
      if (err) {
        return reject(err);
      }

      if (!data.repository || !data.repository.url) {
        return reject(new Error('Missing boilerplate metadata'));
      }

      var gh = parseGitUri(data.repository.url);

      return github.repos.getArchiveLink({
        user: gh.user,
        repo: gh.repo,
        archive_format: 'tarball',
        ref: 'master'
      }, function (err2, res2) {
        if (err2) {
          return reject(err2);
        }

        return resolve(res2.meta.location);
      });
    });
  });
}

function initFromNpm(config) {
  return new Promise(function (resolve, reject) {
    var uri = baseUri + config.boilerplate + '/latest';
    npm.get(uri, params, function (err, data) {
      if (err) {
        return reject(err);
      }

      if (!data.dist || !data.dist.tarball) {
        return reject(new Error('Missing boilerplate metadata'));
      }

      return resolve(data.dist.tarball);
    });
  });
}

function processTarball(console, config, tarballUrl) {
  var directory = config.output;
  var download = new Download({
    extract: true,
    strip: 1,
    mode: '644'
  }).get(tarballUrl).dest(directory);

  console.info('Downloading boilerplate');

  return new Promise(function (resolve, reject) {
    return download.run(function (err) {
      if (err) {
        return reject(err);
      }

      console.info('Downloaded boilerplate');
      console.info('Installing dependencies');

      shell.cd(directory);
      shell.exec('npm install');

      return resolve();
    });
  });
}

function parseGitUri(uri) {
  var parts = uri.replace('.git', '').split('/');

  return {
    repo: parts[parts.length - 1],
    user: parts[parts.length - 2]
  };
}